/*
 * MIT License
 *
 * Copyright (c) 2020 ManBang Group
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package io.manbang.wireless.thresh.jsruntime;

import android.text.TextUtils;

import com.eclipsesource.v8.JavaCallback;
import com.eclipsesource.v8.JavaVoidCallback;
import com.eclipsesource.v8.V8;
import com.eclipsesource.v8.V8Object;
import io.manbang.wireless.thresh.callback.ThreshCallback;
import io.manbang.wireless.thresh.util.ThreshLogger;
import io.manbang.wireless.thresh.util.EnumHostType;
import io.manbang.wireless.thresh.util.FileUtils;

/**
 * js执行器
 */
public class JSExecutor {

    private V8 v8Runtime;
    /**
     * 是否测试模式
     */
    private EnumHostType enumHostType;

    public JSExecutor() {
        ensureInitV8();
    }


    public V8 ensureInitV8() {
        try {
            if (v8Runtime == null) {
                v8Runtime = V8.createV8Runtime();
            }
        } catch (Exception e) {
            ThreshLogger.e("ensureInitV8", e.getMessage());
        }
        return v8Runtime;
    }

    public void withHostType(EnumHostType hostType) {
        this.enumHostType = hostType;
    }

    public boolean isDebug() {
        return enumHostType == EnumHostType.TEST;
    }


    public void registerObj() {
        try {
            V8Object v8Console = new V8Object(v8Runtime);
            v8Runtime.add("console", v8Console);
            CompatInjectJS jsConsole = new CompatInjectJS();
            v8Console.registerJavaMethod(jsConsole, "log", "log", new Class[]{Object.class});
            v8Console.registerJavaMethod(jsConsole, "group", "group", new Class[]{Object.class});
            v8Console.registerJavaMethod(jsConsole, "groupEnd", "groupEnd",
                                         new Class[]{Object.class});
            v8Console.registerJavaMethod(jsConsole, "groupEnd", "groupEnd", null);
            v8Console.close();
            ThreshLogger.i("js_engine_registerObj");
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void registerJavaMethod(JavaVoidCallback callback, String name) {
        try {
            v8Runtime.registerJavaMethod(callback, name);
        } catch (Exception e) {
            e.printStackTrace();
            ThreshLogger.e("js_engine_register_method", e);
        }

    }

    public void registerJavaMethod(JavaCallback callback, String name) {

        try {
            v8Runtime.registerJavaMethod(callback, name);
            ThreshLogger.i("js_engine_register_java_method");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void registerJavaMethod(Object object, String methodName, String jsFunctionName,
                                   Class<?>[] parameterTypes) {

        try {
            v8Runtime.registerJavaMethod(object, methodName, jsFunctionName, parameterTypes);
        } catch (Exception e) {
            e.printStackTrace();
            ThreshLogger.e(null, e);
        }
    }

    /**
     * 执行js
     */
    public Object executeScript(String script, ThreshCallback dynamicCallback) {

        try {
            Object result = v8Runtime.executeScript(script);
            if (dynamicCallback != null && dynamicCallback.dispatchResult()) {
                dynamicCallback.onSuccess(result);
            }
            ThreshLogger.i("js_engine_execute_script");
        } catch (Exception e) {
            e.printStackTrace();
            ThreshLogger.e(null, e);
            if (dynamicCallback != null && dynamicCallback.dispatchResult()) {
                dynamicCallback.onError(-1, e.getMessage());
            }
        }
        return null;
    }

    /**
     * 执行js方法
     */
    public void executeJSFunction(String funcName, ThreshCallback dynamicCallback,
                                  Object... params) {

        try {
            Object result = v8Runtime.executeJSFunction(funcName, params);
            if (dynamicCallback != null && dynamicCallback.dispatchResult()) {
                dynamicCallback.onSuccess(result);
            }
            ThreshLogger.i("js_engine_execute_js_func ：" + funcName);
        } catch (Exception e) {
            ThreshLogger.e(null, e);
            if (dynamicCallback != null && dynamicCallback.dispatchResult()) {
                dynamicCallback.onError(-1, e.getMessage());
            }
        }
    }

    public void close() {
        try {
            if (v8Runtime != null) {
                v8Runtime.close();
                v8Runtime = null;
            }
        } catch (Exception e) {
            ThreshLogger.e(null, e);
        }
    }

    public synchronized void executeScriptFile(String fileName,
                                               ThreshCallback dynamicCallback) {
        try {
            String script = FileUtils.getFromFile(fileName);
            if (TextUtils.isEmpty(script)) {
                if (dynamicCallback != null && dynamicCallback.dispatchResult()) {
                    dynamicCallback.onError(-1, "error path");
                }
                return;
            }
            Object result = v8Runtime.executeScript(script);
            if (dynamicCallback != null && dynamicCallback.dispatchResult()) {
                dynamicCallback.onSuccess(result);
            }
            ThreshLogger.i("js_engine_execute_script_file");
        } catch (Exception e) {
            e.printStackTrace();
            if (dynamicCallback != null && dynamicCallback.dispatchResult()) {
                dynamicCallback.onError(-1, e.getMessage());
            }
        }
    }
}